@@ -758,4 +758,8 @@ public class BufferRow extends ResultSetRow {
 		return getNativeTime(columnIndex, this.rowFromServer.getByteBuffer(),
 				offset, (int) length, targetCalendar, tz, rollForward, conn, rs);
 	}
+
+	public int getBytesSize() {
+		return this.rowFromServer.getBufLength();
+	}
 }
\ No newline at end of file
