@@ -927,10 +927,27 @@ public class SQLError {
 	}
 
 	public static SQLException createSQLException(String message, ExceptionInterceptor interceptor) {
-		return new SQLException(message);
+		return createSQLException(message, interceptor, null);
+	}
+	public static SQLException createSQLException(String message, ExceptionInterceptor interceptor, Connection conn) {
+		SQLException sqlEx = new SQLException(message);
+		
+		if (interceptor != null) {
+			SQLException interceptedEx = interceptor.interceptException(sqlEx, conn);
+			
+			if (interceptedEx != null) {
+				return interceptedEx;
+			}
+		}
+		
+		return sqlEx;
 	}
 
 	public static SQLException createSQLException(String message, String sqlState, Throwable cause, ExceptionInterceptor interceptor) {
+		return createSQLException(message, sqlState, cause, interceptor, null);
+	}
+	public static SQLException createSQLException(String message, String sqlState, Throwable cause, ExceptionInterceptor interceptor,
+			Connection conn) {
 		if (THROWABLE_INIT_CAUSE_METHOD == null) {
 			if (cause != null) {
 				message = message + " due to " + cause.toString();
@@ -948,6 +965,14 @@ public class SQLError {
 			}
 		}
 		
+		if (interceptor != null) {
+			SQLException interceptedEx = interceptor.interceptException(sqlEx, conn);
+			
+			if (interceptedEx != null) {
+				return interceptedEx;
+			}
+		}
+		
 		return sqlEx;
 	}
 	
@@ -958,106 +983,126 @@ public class SQLError {
 	
 	public static SQLException createSQLException(String message,
 			String sqlState, int vendorErrorCode, boolean isTransient, ExceptionInterceptor interceptor) {
+		return createSQLException(message, sqlState, vendorErrorCode, false, interceptor, null);
+	}
+	public static SQLException createSQLException(String message,
+			String sqlState, int vendorErrorCode, boolean isTransient, ExceptionInterceptor interceptor, Connection conn) {
 		try {
+			SQLException sqlEx = null;
+			
 			if (sqlState != null) {
 				if (sqlState.startsWith("08")) {
 					if (isTransient) {
 						if (!Util.isJdbc4()) {
-							return new MySQLTransientConnectionException(
+							sqlEx = new MySQLTransientConnectionException(
 									message, sqlState, vendorErrorCode);
-						}
-
-						return (SQLException) Util
+						} else {
+							sqlEx = (SQLException) Util
 								.getInstance(
 										"com.mysql.jdbc.exceptions.jdbc4.MySQLTransientConnectionException",
 										new Class[] { String.class,
 												String.class, Integer.TYPE },
 										new Object[] { message, sqlState,
 												Constants.integerValueOf(vendorErrorCode) }, interceptor);
-					}
-
-					if (!Util.isJdbc4()) {
-						return new MySQLNonTransientConnectionException(
+						}
+					} else if (!Util.isJdbc4()) {
+						sqlEx = new MySQLNonTransientConnectionException(
 								message, sqlState, vendorErrorCode);
-					}
-
-					return (SQLException) Util
-							.getInstance(
+					} else {
+						sqlEx = (SQLException) Util.getInstance(
 									"com.mysql.jdbc.exceptions.jdbc4.MySQLNonTransientConnectionException",
 									new Class[] { String.class, String.class,
 											Integer.TYPE  }, new Object[] {
 											message, sqlState,
 											Constants.integerValueOf(vendorErrorCode) }, interceptor);
-				}
-
-				if (sqlState.startsWith("22")) {
-					if (!Util.isJdbc4()) {
-						return new MySQLDataException(message, sqlState,
-								vendorErrorCode);
 					}
-
-					return (SQLException) Util
+				} else if (sqlState.startsWith("22")) {
+					if (!Util.isJdbc4()) {
+						sqlEx = new MySQLDataException(message, sqlState,
+								vendorErrorCode);
+					} else {
+						sqlEx = (SQLException) Util
 							.getInstance(
 									"com.mysql.jdbc.exceptions.jdbc4.MySQLDataException",
 									new Class[] { String.class, String.class,
 											Integer.TYPE  }, new Object[] {
 											message, sqlState,
 											Constants.integerValueOf(vendorErrorCode) }, interceptor);
-				}
-
-				if (sqlState.startsWith("23")) {
+					}
+				} else if (sqlState.startsWith("23")) {
 
 					if (!Util.isJdbc4()) {
-						return new MySQLIntegrityConstraintViolationException(
+						sqlEx = new MySQLIntegrityConstraintViolationException(
 								message, sqlState, vendorErrorCode);
-					}
-
-					return (SQLException) Util
+					} else {
+						sqlEx =  (SQLException) Util
 							.getInstance(
 									"com.mysql.jdbc.exceptions.jdbc4.MySQLIntegrityConstraintViolationException",
 									new Class[] { String.class, String.class,
 											Integer.TYPE  }, new Object[] {
 											message, sqlState,
 											Constants.integerValueOf(vendorErrorCode) }, interceptor);
-				}
-
-				if (sqlState.startsWith("42")) {
-					if (!Util.isJdbc4()) {
-						return new MySQLSyntaxErrorException(message, sqlState,
-								vendorErrorCode);
 					}
-
-					return (SQLException) Util
-							.getInstance(
+				} else if (sqlState.startsWith("42")) {
+					if (!Util.isJdbc4()) {
+						sqlEx = new MySQLSyntaxErrorException(message, sqlState,
+								vendorErrorCode);
+					} else {
+						sqlEx = (SQLException) Util.getInstance(
 									"com.mysql.jdbc.exceptions.jdbc4.MySQLSyntaxErrorException",
 									new Class[] { String.class, String.class,
 											Integer.TYPE  }, new Object[] {
 											message, sqlState,
 											Constants.integerValueOf(vendorErrorCode) }, interceptor);
-				}
-
-				if (sqlState.startsWith("40")) {
-					if (!Util.isJdbc4()) {
-						return new MySQLTransactionRollbackException(message,
-								sqlState, vendorErrorCode);
 					}
-
-					return (SQLException) Util
+				} else if (sqlState.startsWith("40")) {
+					if (!Util.isJdbc4()) {
+						sqlEx = new MySQLTransactionRollbackException(message,
+								sqlState, vendorErrorCode);
+					} else {
+						sqlEx = (SQLException) Util
 							.getInstance(
 									"com.mysql.jdbc.exceptions.jdbc4.MySQLTransactionRollbackException",
 									new Class[] { String.class, String.class,
 											Integer.TYPE  }, new Object[] {
 											message, sqlState,
 											Constants.integerValueOf(vendorErrorCode) }, interceptor);
+					}
+				} else {
+					sqlEx = new SQLException(message, sqlState, vendorErrorCode);
+				}
+			} else {
+				sqlEx = new SQLException(message, sqlState, vendorErrorCode);
+			}
+			
+			if (interceptor != null) {
+				SQLException interceptedEx = interceptor.interceptException(sqlEx, conn);
+				
+				if (interceptedEx != null) {
+					return interceptedEx;
 				}
 			}
-
-			return new SQLException(message, sqlState, vendorErrorCode);
+			
+			if (sqlEx == null) {
+				System.out.println("!");
+			}
+			
+			return sqlEx;
 		} catch (SQLException sqlEx) {
-			return new SQLException(
+			SQLException unexpectedEx = new SQLException(
 					"Unable to create correct SQLException class instance, error class/codes may be incorrect. Reason: "
 							+ Util.stackTraceToString(sqlEx),
 					SQL_STATE_GENERAL_ERROR);
+			
+			if (interceptor != null) {
+				SQLException interceptedEx = interceptor.interceptException(unexpectedEx, conn);
+				
+				if (interceptedEx != null) {
+					return interceptedEx;
+				}
+			}
+			
+			return unexpectedEx;
 		}
 	}
 	
@@ -1089,6 +1134,14 @@ public class SQLError {
 			}
 		}
 		
+		if (interceptor != null) {
+			SQLException interceptedEx = interceptor.interceptException(exToReturn, conn);
+			
+			if (interceptedEx != null) {
+				return interceptedEx;
+			}
+		}
+		
 		return exToReturn;
 	}
 	
