@@ -723,7 +723,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 								batchTimeout != 0
 								&& locallyScopedConn.versionMeetsMinimum(5, 0, 0)) {
 							timeoutTask = new CancelTask(this);
-							ConnectionImpl.getCancelTimer().schedule(timeoutTask,
+							locallyScopedConn.getCancelTimer().schedule(timeoutTask,
 									batchTimeout);
 						}
 						
@@ -1173,6 +1173,17 @@ public class ServerPreparedStatement extends PreparedStatement {
 			boolean createStreamingResultSet, 
 			Field[] metadataFromCache) throws SQLException {
 		synchronized (this.connection.getMutex()) {
+			MysqlIO mysql = this.connection.getIO();
+
+			if (mysql.shouldIntercept()) {
+				ResultSetInternalMethods interceptedResults =
+	    			mysql.invokeStatementInterceptorsPre(this.originalSql, this, true);
+	
+	    		if (interceptedResults != null) {
+	    			return interceptedResults;
+	    		}
+			}
+    		
 			if (this.detectedLongParameterSwitch) {
 				// Check when values were bound
 				boolean firstFound = false;
@@ -1226,7 +1237,6 @@ public class ServerPreparedStatement extends PreparedStatement {
 			//
 			// store the parameter values
 			//
-			MysqlIO mysql = this.connection.getIO();
 
 			Buffer packet = mysql.getSharedSendPacket();
 
@@ -1435,6 +1445,15 @@ public class ServerPreparedStatement extends PreparedStatement {
 						this.currentCatalog, resultPacket, true, this.fieldCount,
 						metadataFromCache);
 				
+				if (mysql.shouldIntercept()) {
+					ResultSetInternalMethods interceptedResults =
+		    			mysql.invokeStatementInterceptorsPost(this.originalSql, this, rs, true, null);
+		
+		    		if (interceptedResults != null) {
+		    			rs = interceptedResults;
+		    		}
+				}
+				
 				if (this.profileSQL) {
 					long fetchEndTime = mysql.getCurrentTimeNanosOrMillis();
 
@@ -1468,6 +1487,12 @@ public class ServerPreparedStatement extends PreparedStatement {
 		        }
 				
 				return rs;
+			} catch (SQLException sqlEx) {
+				if (mysql.shouldIntercept()) {
+					mysql.invokeStatementInterceptorsPost(this.originalSql, this, null, true, sqlEx);
+				}
+				
+				throw sqlEx;
 			} finally {
 				if (timeoutTask != null) {
 					timeoutTask.cancel();
